﻿using NUnit.Framework;
using Spring.Context.Support;
using Spring.Context;
using Spring.Objects.Factory.Config;

namespace ClassLibrary1
{
    [TestFixture]
    public class DoSomeWorkTests
    {
        private GenericApplicationContext _context;

        [TestFixtureSetUp]
        public void _TestFixtureSetUp()
        {
            _context = new GenericApplicationContext();

            _context.Configure(new StandardObjectConfiguration(AutoWiringMode.Constructor)
                .DefineObject<DataAccess>()
                .DefineObject<DataPrinter>()
                .DefineObject<DataSorter>()
                .DefineObject<Worker>()
                .DefineObject<ConnectionManager>()
                .DefineObject<LoggerAdvice>()
                );

            _context.Configure(new StandardAdviceConfiguration()
                .DefinePointcut<AllTypesPointcut>().WithAdvice(typeof(LoggerAdvice).FullName)
                );
        }

        [Test]
        public void CanDoWork()
        {
            ConnectionManager connectionManager = new ConnectionManager();
            DataAccess dataAccess = new DataAccess(connectionManager);
            DataPrinter dataPrinter = new DataPrinter();
            DataSorter dataSorter = new DataSorter();

            var worker = new Worker(dataAccess, dataSorter, dataPrinter);

            worker.DoWork();
        }

        [Test]
        public void CanDoWorkEasily()
        {
            var worker = _context.GetObject<Worker>();
            worker.DoWork();

        }

    }
}
