﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Diagnostics;
using AopAlliance.Intercept;
using Spring.Aop.Framework.AutoProxy;

namespace ClassLibrary1
{
    [AttributeUsage(AttributeTargets.Method)]
    public class LogAttribute : System.Attribute
    {

        private string _message;
        public LogAttribute(string message)
        {
            _message = message;

        }
        public string Message
        {
            get
            {
                return _message;
            }
        }
    }

    public class AllTypesPointcut : AbstractFilteringAutoProxyCreator
    {
        protected override bool IsEligibleForProxying(Type targetType, string targetName)
        {
            return true;
        }
    }

    public class LoggerAdvice : IMethodInterceptor
    {

        public object Invoke(IMethodInvocation invocation)
        {
            object[] logAttribs = invocation.Method.GetCustomAttributes(typeof(LogAttribute), true);

            if (logAttribs.Length != 0)
            {
                LogAttribute logAttrib = (LogAttribute)logAttribs[0];

                Debug.WriteLine(String.Format("Advice is Logging Method {0}() with message {1}", invocation.Method.Name, logAttrib.Message));
            }

            return invocation.Proceed();
        }
    }


    public class ConnectionManager
    {

        [Log("Getting Connection String")]
        public virtual string GetConnectionString()
        {
            //get connection string from config file or something...
            return "I am the connection string here!";
        }
    }

    public class DataAccess
    {
        private ConnectionManager _connectionManger;

        /// <summary>
        /// Initializes a new instance of the DataAccess class.
        /// </summary>
        public DataAccess(ConnectionManager connectionManger)
        {
            _connectionManger = connectionManger;
        }

        [Log("Getting Data...")]
        public virtual IEnumerable<string> GetData()
        {
            string connectionString = _connectionManger.GetConnectionString();

            //do something with connection string in here...
            return new string[] { "one", "two", "three", "four" };
        }

    }

    public class DataSorter
    {
        [Log("Sorting data...")]
        public virtual IEnumerable<string> SortData(IEnumerable<string> data)
        {
            var orderedData = from item in data orderby item select item;
            return orderedData;

        }
    }

    public class DataPrinter
    {
        [Log("Printing Data...")]
        public virtual void PrintData(IEnumerable<string> data)
        {
            foreach (string item in data)
            {
                Debug.WriteLine(item);
            }
        }
    }

    public class Worker
    {
        private DataAccess _dataAccess;

        private DataPrinter _dataPrinter;

        private DataSorter _dataSorter;

        /// <summary>
        /// Initializes a new instance of the Worker class.
        /// </summary>
        public Worker(DataAccess dataAccess, DataSorter dataSorter, DataPrinter dataPrinter)
        {
            _dataPrinter = dataPrinter;
            _dataSorter = dataSorter;
            _dataAccess = dataAccess;

        }

        [Log("Doing the work...")]
        public virtual void DoWork()
        {
            IEnumerable<string> data = _dataAccess.GetData();

            data = _dataSorter.SortData(data);

            _dataPrinter.PrintData(data);

        }

    }
}
